@@ -1,20 +1,17 @@
 """
-Class and standalone app for Independent Component Analysis
+Class for Independent Component Analysis
 """
 
-import argparse
-from numpy import random, sqrt, zeros, real, dot, outer, diag, transpose
-from scipy.linalg import sqrtm, inv, orth
-from thunder.utils import ThunderContext
-from thunder.utils import save
-from thunder.factorization import SVD
-from thunder.utils.matrices import RowMatrix
+from thunder.factorization.svd import SVD
+from thunder.rdds.series import Series
+from thunder.rdds.matrices import RowMatrix
 
 
 class ICA(object):
     """
-    Independent component analysis on a dense matrix
-    with initial dimensionality reduction via SVD
+    Independent component analysis on a distributed matrix.
+
+    Initial dimensionality reduction performed via SVD
 
     Parameters
     ----------
@@ -41,8 +38,9 @@ class ICA(object):
     `a` : array, shape (ncols, ncols)
         Recovered mixing matrix
 
-    `sigs` : RDD of nrows (tuple, array) pairs, each array of shape (c,)
+    `sigs` : RowMatrix, nrows, each array of shape (c,)
         Estimated independent components
+
     """
 
     def __init__(self, c, k=None, svdmethod="direct", maxiter=10, tol=0.000001, seed=0):
@@ -59,15 +57,26 @@ class ICA(object):
 
         Parameters
         ----------
-        data: RDD of (tuple, array) pairs, or RowMatrix
-            Data to estimate independent components from
+        data : Series or a subclass (e.g. RowMatrix)
+            Data to estimate independent components from, must be a collection of
+            key-value pairs where the keys are identifiers and the values are
+            one-dimensional arrays
 
         Returns
         ----------
         self : returns an instance of self.
         """
 
-        d = len(data.first()[1])
+        from numpy import random, sqrt, zeros, real, dot, outer, diag, transpose
+        from scipy.linalg import sqrtm, inv, orth
+
+        if not (isinstance(data, Series)):
+            raise Exception('Input must be Series or a subclass (e.g. RowMatrix)')
+
+        if not isinstance(data, RowMatrix):
+            data = data.toRowMatrix()
+
+        d = data.ncols
 
         if self.k is None:
             self.k = d
@@ -80,9 +89,6 @@ class ICA(object):
             raise Exception("number of principal comps " + str(self.k) +
                             " must be less than the data dimensionality " + str(d))
 
-        if type(data) is not RowMatrix:
-            data = RowMatrix(data)
-
         # reduce dimensionality
         svd = SVD(k=self.k, method=self.svdmethod).calc(data)
 
@@ -112,6 +118,7 @@ class ICA(object):
             b_old = b
             errvec[iter-1] = (1 - minabscos)
 
+
         # get un-mixing matrix
         w = dot(b.T, whtmat)
 
@@ -119,35 +126,10 @@ class ICA(object):
         a = dot(unwhtmat, b)
 
         # get components
-        sigs = data.times(w.T).rdd
+        sigs = data.times(w.T)
 
         self.w = w
         self.a = a
         self.sigs = sigs
 
         return self
-
-
-if __name__ == "__main__":
-    parser = argparse.ArgumentParser(description="do independent components analysis")
-    parser.add_argument("datafile", type=str)
-    parser.add_argument("outputdir", type=str)
-    parser.add_argument("k", type=int)
-    parser.add_argument("c", type=int)
-    parser.add_argument("--svdmethod", choices=("direct", "em"), default="direct", required=False)
-    parser.add_argument("--maxiter", type=float, default=100, required=False)
-    parser.add_argument("--tol", type=float, default=0.000001, required=False)
-    parser.add_argument("--seed", type=int, default=0, required=False)
-    parser.add_argument("--preprocess", choices=("raw", "dff", "sub", "dff-highpass", "dff-percentile"
-                        "dff-detrendnonlin", "dff-detrend-percentile"), default="raw", required=False)
-
-    args = parser.parse_args()
-    
-    tsc = ThunderContext.start(appName="ica")
-
-    data = tsc.loadText(args.datafile, args.preprocess).cache()
-    result = ICA(args.k, args.c, args.svdmethod, args.maxiter, args.tol, args.seed).fit(data)
-
-    outputdir = args.outputdir + "-ica"
-    save(result.w, outputdir, "w", "matlab")
-    save(result.sigs, outputdir, "sigs", "matlab")
