@@ -2,16 +2,15 @@
 Class for performing Singular Value Decomposition
 """
 
-from numpy import random, sum, argsort, dot, outer, sqrt
-from scipy.linalg import inv, orth
-from numpy.linalg import eigh
-from thunder.utils import RowMatrix
+from numpy import zeros, shape
+
+from thunder.rdds.series import Series
+from thunder.rdds.matrices import RowMatrix
 
 
 class SVD(object):
     """
-    Large-scale singular value decomposiiton on a dense matrix
-    represented as an RDD or RowMatrix with nrows and ncols
+    Singular value decomposiiton on a distributed matrix.
 
     Parameters
     ----------
@@ -29,7 +28,7 @@ class SVD(object):
 
     Attributes
     ----------
-    `u` : RDD of nrows (tuple, array) pairs, each array of shape (k,)
+    `u` : RowMatrix, nrows, each of shape (k,)
         Left singular vectors
 
     `s` : array, shape(nrows,)
@@ -50,15 +49,23 @@ class SVD(object):
 
         Parameters
         ----------
-        mat : RDD of (tuple, array) pairs, or RowMatrix
+        mat :  Series or a subclass (e.g. RowMatrix)
             Matrix to compute singular vectors from
 
         Returns
         ----------
         self : returns an instance of self.
         """
-        if type(mat) is not RowMatrix:
-            mat = RowMatrix(mat)
+
+        from numpy import random, sum, argsort, dot, outer, sqrt
+        from scipy.linalg import inv, orth
+        from numpy.linalg import eigh
+
+        if not (isinstance(mat, Series)):
+            raise Exception('Input must be Series or a subclass (e.g. RowMatrix)')
+
+        if not (isinstance(mat, RowMatrix)):
+            mat = mat.toRowMatrix()
 
         if self.method == "direct":
 
@@ -74,7 +81,7 @@ class SVD(object):
             # project back into data, normalize by singular values
             u = mat.times(v.T / s)
 
-            self.u = u.rdd
+            self.u = u
             self.s = s
             self.v = v
 
@@ -85,20 +92,48 @@ class SVD(object):
             iter = 0
             error = 100
 
+            # define an accumulator
+            from pyspark.accumulators import AccumulatorParam
+
+            class MatrixAccumulatorParam(AccumulatorParam):
+                def zero(self, value):
+                    return zeros(shape(value))
+
+                def addInPlace(self, val1, val2):
+                    val1 += val2
+                    return val1
+
+            # define an accumulator function
+            global runsum
+
+            def outerSumOther(x, y):
+                global runsum
+                runsum += outer(x, dot(x, y))
+
             # iterative update subspace using expectation maximization
             # e-step: x = (c'c)^-1 c' y
             # m-step: c = y x' (xx')^-1
             while (iter < self.maxiter) & (error > self.tol):
+
                 c_old = c
+
                 # pre compute (c'c)^-1 c'
                 c_inv = dot(c.T, inv(dot(c, c.T)))
+
                 # compute (xx')^-1 through a map reduce
                 xx = mat.times(c_inv).gramian()
                 xx_inv = inv(xx)
+
                 # pre compute (c'c)^-1 c' (xx')^-1
                 premult2 = mat.rdd.context.broadcast(dot(c_inv, xx_inv))
-                # compute the new c through a map reduce
-                c = mat.rows().map(lambda x: outer(x, dot(x, premult2.value))).sum()
+
+                # compute the new c using an accumulator
+                # direct approach: c = mat.rows().map(lambda x: outer(x, dot(x, premult2.value))).sum()
+                runsum = mat.rdd.context.accumulator(zeros((mat.ncols, self.k)), MatrixAccumulatorParam())
+                mat.rows().foreach(lambda x: outerSumOther(x, premult2.value))
+                c = runsum.value
+
+                # transpose result
                 c = c.T
 
                 error = sum(sum((c - c_old) ** 2))
@@ -114,7 +149,7 @@ class SVD(object):
             v = dot(eigv[:, inds[0:self.k]].T, c.T)
             u = mat.times(v.T / s)
 
-            self.u = u.rdd
+            self.u = u
             self.s = s
             self.v = v
 
